@@ -71,24 +71,25 @@ extern "C" {
 
 #define EcsIdExclusive                 (1u << 9)
 #define EcsIdTraversable               (1u << 10)
-#define EcsIdTag                       (1u << 11)
+#define EcsIdPairIsTag                 (1u << 11)
 #define EcsIdWith                      (1u << 12)
 #define EcsIdCanToggle                 (1u << 13)
 #define EcsIdIsTransitive              (1u << 14)
-#define EcsIdIsInheritable             (1u << 15)
+#define EcsIdInheritable             (1u << 15)
 
 #define EcsIdHasOnAdd                  (1u << 16) /* Same values as table flags */
 #define EcsIdHasOnRemove               (1u << 17) 
 #define EcsIdHasOnSet                  (1u << 18)
-#define EcsIdHasOnTableCreate          (1u << 21)
-#define EcsIdHasOnTableDelete          (1u << 22)
-#define EcsIdIsSparse                  (1u << 23)
-#define EcsIdDontFragment              (1u << 24)
-#define EcsIdMatchDontFragment         (1u << 25) /* For (*, T) wildcards */
-#define EcsIdOrderedChildren           (1u << 28)
+#define EcsIdHasOnTableCreate          (1u << 19)
+#define EcsIdHasOnTableDelete          (1u << 20)
+#define EcsIdSparse                  (1u << 21)
+#define EcsIdDontFragment              (1u << 22)
+#define EcsIdMatchDontFragment         (1u << 23) /* For (*, T) wildcards */
+#define EcsIdOrderedChildren           (1u << 24)
+#define EcsIdSingleton                 (1u << 25)
 #define EcsIdEventMask\
     (EcsIdHasOnAdd|EcsIdHasOnRemove|EcsIdHasOnSet|\
-        EcsIdHasOnTableCreate|EcsIdHasOnTableDelete|EcsIdIsSparse|\
+        EcsIdHasOnTableCreate|EcsIdHasOnTableDelete|EcsIdSparse|\
         EcsIdOrderedChildren)
 
 #define EcsIdMarkedForDelete           (1u << 30)
@@ -218,34 +219,33 @@ extern "C" {
 #define EcsTableHasBuiltins            (1u << 1u)  /* Does table have builtin components */
 #define EcsTableIsPrefab               (1u << 2u)  /* Does the table store prefabs */
 #define EcsTableHasIsA                 (1u << 3u)  /* Does the table have IsA relationship */
-#define EcsTableHasChildOf             (1u << 4u)  /* Does the table type ChildOf relationship */
-#define EcsTableHasName                (1u << 5u)  /* Does the table type have (Identifier, Name) */
-#define EcsTableHasPairs               (1u << 6u)  /* Does the table type have pairs */
-#define EcsTableHasModule              (1u << 7u)  /* Does the table have module data */
-#define EcsTableIsDisabled             (1u << 8u)  /* Does the table type has EcsDisabled */
-#define EcsTableNotQueryable           (1u << 9u)  /* Table should never be returned by queries */
-#define EcsTableHasCtors               (1u << 10u)
-#define EcsTableHasDtors               (1u << 11u)
-#define EcsTableHasCopy                (1u << 12u)
-#define EcsTableHasMove                (1u << 13u)
-#define EcsTableHasToggle              (1u << 14u)
-#define EcsTableHasOverrides           (1u << 15u)
+#define EcsTableHasMultiIsA            (1u << 4u)  /* Does table have multiple IsA pairs */
+#define EcsTableHasChildOf             (1u << 5u)  /* Does the table type ChildOf relationship */
+#define EcsTableHasName                (1u << 6u)  /* Does the table type have (Identifier, Name) */
+#define EcsTableHasPairs               (1u << 7u)  /* Does the table type have pairs */
+#define EcsTableHasModule              (1u << 8u)  /* Does the table have module data */
+#define EcsTableIsDisabled             (1u << 9u)  /* Does the table type has EcsDisabled */
+#define EcsTableNotQueryable           (1u << 10u)  /* Table should never be returned by queries */
+#define EcsTableHasCtors               (1u << 11u)
+#define EcsTableHasDtors               (1u << 12u)
+#define EcsTableHasCopy                (1u << 13u)
+#define EcsTableHasMove                (1u << 14u)
+#define EcsTableHasToggle              (1u << 15u)
 
 #define EcsTableHasOnAdd               (1u << 16u) /* Same values as id flags */
 #define EcsTableHasOnRemove            (1u << 17u)
 #define EcsTableHasOnSet               (1u << 18u)
-#define EcsTableHasOnTableFill         (1u << 19u)
-#define EcsTableHasOnTableEmpty        (1u << 20u)
-#define EcsTableHasOnTableCreate       (1u << 21u)
-#define EcsTableHasOnTableDelete       (1u << 22u)
-#define EcsTableHasSparse              (1u << 23u)
-#define EcsTableHasDontFragment        (1u << 24u)
-#define EcsTableOverrideDontFragment   (1u << 25u)
+#define EcsTableHasOnTableCreate       (1u << 19u)
+#define EcsTableHasOnTableDelete       (1u << 20u)
+#define EcsTableHasSparse              (1u << 21u)
+#define EcsTableHasDontFragment        (1u << 22u)
+#define EcsTableOverrideDontFragment   (1u << 23u)
+#define EcsTableHasOrderedChildren     (1u << 24u)
+#define EcsTableHasOverrides           (1u << 25u)
 
 #define EcsTableHasTraversable         (1u << 27u)
-#define EcsTableHasOrderedChildren     (1u << 28u)
-#define EcsTableEdgeReparent           (1u << 29u)
-#define EcsTableMarkedForDelete        (1u << 30u)
+#define EcsTableEdgeReparent           (1u << 28u)
+#define EcsTableMarkedForDelete        (1u << 29u)
 
 /* Composite table flags */
 #define EcsTableHasLifecycle     (EcsTableHasCtors | EcsTableHasDtors)
