@@ -181,6 +181,7 @@ void SystemMisc_register_callback_after_run(void);
 void SystemMisc_register_run_after_callback(void);
 void SystemMisc_register_callback_after_run_ctx(void);
 void SystemMisc_register_run_after_callback_ctx(void);
+void SystemMisc_run_w_query_next(void);
 
 // Testsuite 'SystemPeriodic'
 void SystemPeriodic_1_type_1_component(void);
@@ -327,6 +328,18 @@ void Stats_get_not_alive_entity_count(void);
 void Stats_progress_stats_systems(void);
 void Stats_progress_stats_systems_w_empty_table_flag(void);
 
+// Testsuite 'Memory'
+void Memory_query_memory_no_cache(void);
+void Memory_query_memory_trivial_cache(void);
+void Memory_query_memory_non_trivial_cache(void);
+void Memory_query_memory_with_groups(void);
+void Memory_query_memory_with_variables(void);
+void Memory_query_memory_with_monitors(void);
+void Memory_commands_memory(void);
+void Memory_table_memory_histogram(void);
+void Memory_sparse_component_memory(void);
+void Memory_sparse_tag_memory(void);
+
 // Testsuite 'Run'
 void Run_setup(void);
 void Run_run(void);
@@ -426,6 +439,7 @@ void Modules_import_monitor_2_worlds(void);
 void Modules_import_monitor_after_mini(void);
 void Modules_import_2_worlds(void);
 void Modules_component_parent_becomes_module(void);
+void Modules_module_has_singleton(void);
 
 // Testsuite 'App'
 void App_app_w_frame_action(void);
@@ -450,6 +464,7 @@ void Rest_try_query(void);
 void Rest_query(void);
 void Rest_named_query(void);
 void Rest_tables(void);
+void Rest_components(void);
 void Rest_request_commands(void);
 void Rest_request_commands_2_syncs(void);
 void Rest_request_commands_no_frames(void);
@@ -1219,6 +1234,10 @@ bake_test_case SystemMisc_testcases[] = {
     {
         "register_run_after_callback_ctx",
         SystemMisc_register_run_after_callback_ctx
+    },
+    {
+        "run_w_query_next",
+        SystemMisc_run_w_query_next
     }
 };
 
@@ -1744,6 +1763,49 @@ bake_test_case Stats_testcases[] = {
     }
 };
 
+bake_test_case Memory_testcases[] = {
+    {
+        "query_memory_no_cache",
+        Memory_query_memory_no_cache
+    },
+    {
+        "query_memory_trivial_cache",
+        Memory_query_memory_trivial_cache
+    },
+    {
+        "query_memory_non_trivial_cache",
+        Memory_query_memory_non_trivial_cache
+    },
+    {
+        "query_memory_with_groups",
+        Memory_query_memory_with_groups
+    },
+    {
+        "query_memory_with_variables",
+        Memory_query_memory_with_variables
+    },
+    {
+        "query_memory_with_monitors",
+        Memory_query_memory_with_monitors
+    },
+    {
+        "commands_memory",
+        Memory_commands_memory
+    },
+    {
+        "table_memory_histogram",
+        Memory_table_memory_histogram
+    },
+    {
+        "sparse_component_memory",
+        Memory_sparse_component_memory
+    },
+    {
+        "sparse_tag_memory",
+        Memory_sparse_tag_memory
+    }
+};
+
 bake_test_case Run_testcases[] = {
     {
         "run",
@@ -2105,6 +2167,10 @@ bake_test_case Modules_testcases[] = {
     {
         "component_parent_becomes_module",
         Modules_component_parent_becomes_module
+    },
+    {
+        "module_has_singleton",
+        Modules_module_has_singleton
     }
 };
 
@@ -2187,6 +2253,10 @@ bake_test_case Rest_testcases[] = {
         "tables",
         Rest_tables
     },
+    {
+        "components",
+        Rest_components
+    },
     {
         "request_commands",
         Rest_request_commands
@@ -2567,7 +2637,7 @@ static bake_test_suite suites[] = {
         "SystemMisc",
         NULL,
         NULL,
-        69,
+        70,
         SystemMisc_testcases
     },
     {
@@ -2640,6 +2710,13 @@ static bake_test_suite suites[] = {
         13,
         Stats_testcases
     },
+    {
+        "Memory",
+        NULL,
+        NULL,
+        10,
+        Memory_testcases
+    },
     {
         "Run",
         Run_setup,
@@ -2669,7 +2746,7 @@ static bake_test_suite suites[] = {
         "Modules",
         Modules_setup,
         NULL,
-        24,
+        25,
         Modules_testcases
     },
     {
@@ -2690,7 +2767,7 @@ static bake_test_suite suites[] = {
         "Rest",
         NULL,
         NULL,
-        21,
+        22,
         Rest_testcases
     },
     {
@@ -2710,5 +2787,5 @@ static bake_test_suite suites[] = {
 };
 
 int main(int argc, char *argv[]) {
-    return bake_test_run("addons", argc, argv, suites, 22);
+    return bake_test_run("addons", argc, argv, suites, 23);
 }
