@@ -63,6 +63,8 @@ void Sparse_count_of_null(void);
 void Sparse_try_low_after_ensure_high(void);
 void Sparse_is_alive_low_after_ensure_high(void);
 void Sparse_remove_low_after_ensure_high(void);
+void Sparse_recreate_pages_after_shrink(void);
+void Sparse_create_low_page_after_high(void);
 
 // Testsuite 'Strbuf'
 void Strbuf_setup(void);
@@ -309,6 +311,14 @@ bake_test_case Sparse_testcases[] = {
     {
         "remove_low_after_ensure_high",
         Sparse_remove_low_after_ensure_high
+    },
+    {
+        "recreate_pages_after_shrink",
+        Sparse_recreate_pages_after_shrink
+    },
+    {
+        "create_low_page_after_high",
+        Sparse_create_low_page_after_high
     }
 };
 
@@ -475,7 +485,7 @@ static bake_test_suite suites[] = {
         "Sparse",
         Sparse_setup,
         NULL,
-        21,
+        23,
         Sparse_testcases
     },
     {
