@@ -652,3 +652,47 @@ void Reference_ref_after_shrink(void) {
 
     ecs_fini(world);
 }
+
+void Reference_ref_after_shrink_w_freed_pages(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    int32_t COUNT = 5000;
+    
+    ecs_entity_t *entities = ecs_os_malloc_n(ecs_entity_t, COUNT);
+    ecs_ref_t *refs = ecs_os_malloc_n(ecs_ref_t, COUNT);
+
+    for (int i = 0; i < COUNT; i ++) {
+        entities[i] = ecs_insert(world, ecs_value(Position, {10 + i, 20}));
+        refs[i] = ecs_ref_init(world, entities[i], Position);
+    }
+
+    for (int i = 0; i < COUNT; i ++) {
+        Position *p = ecs_ref_get(world, &refs[i], Position);
+        test_int(p->x, 10 + i);
+        test_int(p->y, 20);
+    }
+
+    for (int i = 1; i < COUNT; i ++) {
+        ecs_delete(world, entities[i]);
+    }
+
+    ecs_shrink(world);
+
+    {
+        Position *p = ecs_ref_get(world, &refs[0], Position);
+        test_assert(p != NULL);
+        test_int(p->x, 10);
+        test_int(p->y, 20);
+    }
+
+    for (int i = 1; i < COUNT; i ++) {
+        test_assert(ecs_ref_get(world, &refs[i], Position) == NULL);
+    }
+
+    ecs_os_free(entities);
+    ecs_os_free(refs);
+
+    ecs_fini(world);
+}
