@@ -1200,6 +1200,626 @@ void World_delete_empty_tables_after_init(void) {
     ecs_fini(world);
 }
 
+static int dummy_hook_invoked = 0;
+
+static void DummyHook(ecs_iter_t *it) {
+    Position *p = ecs_field(it, Position, 0);
+    test_assert(p != NULL);
+    dummy_hook_invoked ++;
+}
+
+void World_delete_empty_tables_w_add_hook_clear(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_add = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_entity_t p = ecs_new(world);
+    ecs_set(world, p, Position, {10, 20});
+    ecs_delete(world, p);
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_fini(world);
+}
+
+void World_delete_empty_tables_w_add_hook_delete(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_add = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_entity_t p = ecs_new(world);
+    ecs_set(world, p, Position, {10, 20});
+    ecs_delete(world, p);
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_fini(world);
+}
+
+void World_delete_empty_tables_w_add_hook_clear_delete(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_add = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_entity_t p = ecs_new(world);
+    ecs_set(world, p, Position, {10, 20});
+    ecs_delete(world, p);
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1,
+        .delete_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1,
+        .delete_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1,
+        .delete_generation = 1
+    });
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_fini(world);
+}
+
+void World_delete_empty_tables_w_add_hook_clear_before_delete(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_add = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_entity_t p = ecs_new(world);
+    ecs_set(world, p, Position, {10, 20});
+    ecs_delete(world, p);
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1,
+        .delete_generation = 2
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1,
+        .delete_generation = 2
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1,
+        .delete_generation = 2
+    });
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_fini(world);
+}
+
+void World_delete_empty_tables_w_remove_hook_clear(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_remove = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_entity_t p = ecs_new(world);
+    ecs_set(world, p, Position, {10, 20});
+    ecs_delete(world, p);
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_fini(world);
+
+    test_int(1, dummy_hook_invoked);
+}
+
+void World_delete_empty_tables_w_remove_hook_delete(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_remove = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_entity_t p = ecs_new(world);
+    ecs_set(world, p, Position, {10, 20});
+    ecs_delete(world, p);
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_fini(world);
+
+    test_int(1, dummy_hook_invoked);
+}
+
+void World_delete_empty_tables_w_remove_hook_clear_delete(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_remove = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_entity_t p = ecs_new(world);
+    ecs_set(world, p, Position, {10, 20});
+    ecs_delete(world, p);
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1,
+        .delete_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1,
+        .delete_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1,
+        .delete_generation = 1
+    });
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_fini(world);
+
+    test_int(1, dummy_hook_invoked);
+}
+
+void World_delete_empty_tables_w_remove_hook_clear_before_delete(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_remove = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_entity_t p = ecs_new(world);
+    ecs_set(world, p, Position, {10, 20});
+    ecs_delete(world, p);
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1,
+        .delete_generation = 2
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1,
+        .delete_generation = 2
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1,
+        .delete_generation = 2
+    });
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_fini(world);
+
+    test_int(1, dummy_hook_invoked);
+}
+
+void World_delete_empty_tables_w_set_hook_clear(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_set = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_entity_t p = ecs_new(world);
+    ecs_set(world, p, Position, {10, 20});
+    ecs_delete(world, p);
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_fini(world);
+
+    test_int(1, dummy_hook_invoked);
+}
+
+void World_delete_empty_tables_w_set_hook_delete(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_set = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_entity_t p = ecs_new(world);
+    ecs_set(world, p, Position, {10, 20});
+    ecs_delete(world, p);
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_fini(world);
+
+    test_int(1, dummy_hook_invoked);
+}
+
+void World_delete_empty_tables_w_set_hook_clear_delete(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_set = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_entity_t p = ecs_new(world);
+    ecs_set(world, p, Position, {10, 20});
+    ecs_delete(world, p);
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1,
+        .clear_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1,
+        .clear_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1,
+        .clear_generation = 1
+    });
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_fini(world);
+
+    test_int(1, dummy_hook_invoked);
+}
+
+void World_delete_empty_tables_w_set_hook_clear_before_delete(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_set = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_entity_t p = ecs_new(world);
+    ecs_set(world, p, Position, {10, 20});
+    ecs_delete(world, p);
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 2,
+        .clear_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 2,
+        .clear_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 2,
+        .clear_generation = 1
+    });
+
+    test_int(1, dummy_hook_invoked);
+
+    ecs_fini(world);
+
+    test_int(1, dummy_hook_invoked);
+}
+
+void World_delete_empty_tables_w_add_hook_delete_empty_table(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_set = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_table_t *t = ecs_table_find(world, &ecs_id(Position), 1);
+    test_assert(t != NULL);
+    test_int(ecs_table_count(t), 0);
+    test_int(ecs_table_size(t), 0);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_fini(world);
+}
+
+void World_delete_empty_tables_w_remove_hook_delete_empty_table(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_remove = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_table_t *t = ecs_table_find(world, &ecs_id(Position), 1);
+    test_assert(t != NULL);
+    test_int(ecs_table_count(t), 0);
+    test_int(ecs_table_size(t), 0);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_fini(world);
+}
+
+void World_delete_empty_tables_w_set_hook_delete_empty_table(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_set = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_table_t *t = ecs_table_find(world, &ecs_id(Position), 1);
+    test_assert(t != NULL);
+    test_int(ecs_table_count(t), 0);
+    test_int(ecs_table_size(t), 0);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .clear_generation = 1
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_fini(world);
+}
+
+void World_delete_empty_tables_w_add_hook_clear_empty_table(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_add = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_table_t *t = ecs_table_find(world, &ecs_id(Position), 1);
+    test_assert(t != NULL);
+    test_int(ecs_table_count(t), 0);
+    test_int(ecs_table_size(t), 0);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_fini(world);
+}
+
+void World_delete_empty_tables_w_remove_hook_clear_empty_table(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_remove = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_table_t *t = ecs_table_find(world, &ecs_id(Position), 1);
+    test_assert(t != NULL);
+    test_int(ecs_table_count(t), 0);
+    test_int(ecs_table_size(t), 0);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_fini(world);
+}
+
+void World_delete_empty_tables_w_set_hook_clear_empty_table(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_set_hooks(world, Position, {
+        .on_set = DummyHook
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_table_t *t = ecs_table_find(world, &ecs_id(Position), 1);
+    test_assert(t != NULL);
+    test_int(ecs_table_count(t), 0);
+    test_int(ecs_table_size(t), 0);
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    ecs_delete_empty_tables(world, &(ecs_delete_empty_tables_desc_t) {
+        .delete_generation = 1
+    });
+
+    test_int(0, dummy_hook_invoked);
+
+    ecs_fini(world);
+}
+
 void World_delete_1000_empty_tables(void) {
     ecs_world_t *world = ecs_mini();
 
@@ -2050,6 +2670,45 @@ void World_init_create_delete_random_2_entities_shrink_fini(void) {
     ecs_fini(world);
 }
 
+void World_recreate_tables_after_shrink(void) {
+    ecs_world_t *world = ecs_mini();
+
+    int32_t COUNT = 1000;
+
+    ecs_entity_t *tags = ecs_os_malloc_n(ecs_entity_t, COUNT);
+    for (int32_t i = 0; i < COUNT; i ++) {
+        tags[i] = ecs_new(world);
+    }
+
+    ecs_entity_t e = ecs_new(world);
+    for (int32_t i = 0; i < COUNT; i ++) {
+        ecs_add_id(world, e, tags[i]);
+        if (i) {
+            ecs_remove_id(world, e, tags[i - 1]);
+        }
+    }
+
+    ecs_delete(world, e);
+
+    ecs_shrink(world);
+
+    e = ecs_new(world);
+    for (int32_t i = 0; i < COUNT; i ++) {
+        ecs_add_id(world, e, tags[i]);
+        if (i) {
+            ecs_remove_id(world, e, tags[i - 1]);
+        }
+    }
+
+    test_assert(ecs_is_alive(world, e));
+    test_assert(ecs_get_type(world, e)->count == 1);
+    test_assert(ecs_has_id(world, e, tags[COUNT - 1]));
+
+    ecs_os_free(tags);
+
+    ecs_fini(world);
+}
+
 void World_mini_all_tables_builtin(void) {
     ecs_world_t *world = ecs_mini();
 
@@ -2307,3 +2966,247 @@ void World_user_entity_w_flecs_parent(void) {
 
     ecs_fini(world);
 }
+
+void World_add_exclusive_after_query(void) {
+    install_test_abort();
+
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Position) }}
+    });
+
+    test_assert(q != NULL);
+
+    test_expect_abort();
+
+    ecs_add_id(world, ecs_id(Position), EcsExclusive);
+}
+
+void World_add_with_after_query(void) {
+    install_test_abort();
+
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+    ECS_TAG(world, Foo);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Position) }}
+    });
+
+    test_assert(q != NULL);
+
+    test_expect_abort();
+
+    ecs_add_pair(world, ecs_id(Position), EcsWith, Foo);
+}
+
+void World_add_final_after_query(void) {
+    install_test_abort();
+
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Position) }}
+    });
+
+    test_assert(q != NULL);
+
+    test_expect_abort();
+
+    ecs_add_id(world, ecs_id(Position), EcsFinal);
+}
+
+void World_add_isa_after_query(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+    ECS_TAG(world, Foo);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Position) }}
+    });
+
+    test_assert(q != NULL);
+
+    ecs_add_pair(world, ecs_id(Position), EcsIsA, Foo);
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void World_add_isa_after_query_tgt(void) {
+    install_test_abort();
+
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+    ECS_COMPONENT(world, Velocity);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Velocity) }}
+    });
+
+    test_assert(q != NULL);
+
+    test_expect_abort();
+
+    ecs_add_pair(world, ecs_id(Position), EcsIsA, ecs_id(Velocity));
+}
+
+void World_add_inheritable_after_query(void) {
+    install_test_abort();
+
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Position) }}
+    });
+
+    test_assert(q != NULL);
+
+    test_expect_abort();
+
+    ecs_add_id(world, ecs_id(Position), EcsInheritable);
+}
+
+void World_add_isa_after_query_after_inheritable(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+    ECS_TAG(world, Foo);
+
+    ecs_add_id(world, ecs_id(Position), EcsInheritable);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Position) }}
+    });
+
+    test_assert(q != NULL);
+
+    ecs_add_pair(world, Foo, EcsIsA, ecs_id(Position));
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void World_add_isa_after_query_after_isa(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+    ECS_TAG(world, Foo);
+    ECS_TAG(world, Bar);
+
+    ecs_add_pair(world, ecs_id(Position), EcsIsA, Foo);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Position) }}
+    });
+
+    test_assert(q != NULL);
+
+    ecs_add_pair(world, ecs_id(Position), EcsIsA, Bar);
+
+    ecs_query_fini(q);
+
+    ecs_fini(world);
+}
+
+void World_add_on_instantiate_inherit_after_query(void) {
+    install_test_abort();
+
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Position) }}
+    });
+
+    test_assert(q != NULL);
+
+    test_expect_abort();
+
+    ecs_add_pair(world, ecs_id(Position), EcsOnInstantiate, EcsInherit);
+}
+
+void World_add_sparse_after_query(void) {
+    install_test_abort();
+
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Position) }}
+    });
+
+    test_assert(q != NULL);
+
+    test_expect_abort();
+
+    ecs_add_id(world, ecs_id(Position), EcsSparse);
+}
+
+void World_add_dont_fragment_after_query(void) {
+    install_test_abort();
+
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Position) }}
+    });
+
+    test_assert(q != NULL);
+
+    test_expect_abort();
+
+    ecs_add_id(world, ecs_id(Position), EcsDontFragment);
+}
+
+void World_add_can_toggle_after_query(void) {
+    install_test_abort();
+
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Position) }}
+    });
+
+    test_assert(q != NULL);
+
+    test_expect_abort();
+
+    ecs_add_id(world, ecs_id(Position), EcsCanToggle);
+}
+
+void World_add_traversable_after_query(void) {
+    install_test_abort();
+
+    ecs_world_t *world = ecs_mini();
+
+    ECS_COMPONENT(world, Position);
+
+    ecs_query_t *q = ecs_query(world, {
+        .terms = {{ ecs_id(Position) }}
+    });
+
+    test_assert(q != NULL);
+
+    test_expect_abort();
+
+    ecs_add_id(world, ecs_id(Position), EcsTraversable);
+}
