@@ -2220,3 +2220,280 @@ void Misc_member_metric_w_pair_r_T(void) {
 
     test_int(count, 2);
 }
+
+void Misc_get_const_var(void) {
+    flecs::world world;
+
+    world.script()
+        .code("export const x: 10")
+        .run();
+
+    double v = world.get_const_var<double>("x");
+    test_int(v, 10);
+}
+
+void Misc_get_const_var_struct(void) {
+    flecs::world world;
+
+    world.component<Position>()
+        .member("x", &Position::x)
+        .member("y", &Position::y);
+
+    world.script()
+        .code("export const x = Position: {10, 20}")
+        .run();
+
+    Position v = world.get_const_var<Position>("x");
+    test_int(v.x, 10);
+    test_int(v.y, 20);
+}
+
+void Misc_get_const_var_as_f32(void) {
+    flecs::world world;
+
+    world.script()
+        .code("export const x: 10")
+        .run();
+
+    float v = world.get_const_var<float>("x");
+    test_int(v, 10);
+}
+
+void Misc_get_const_var_as_i32(void) {
+    flecs::world world;
+
+    world.script()
+        .code("export const x: 10")
+        .run();
+
+    int32_t v = world.get_const_var<int32_t>("x");
+    test_int(v, 10);
+}
+
+void Misc_get_const_var_as_u32(void) {
+    flecs::world world;
+
+    world.script()
+        .code("export const x: 10")
+        .run();
+
+    uint32_t v = world.get_const_var<uint32_t>("x");
+    test_uint(v, 10);
+}
+
+void Misc_get_const_var_w_out(void) {
+    flecs::world world;
+
+    world.script()
+        .code("export const x: 10")
+        .run();
+
+    double v;
+    world.get_const_var("x", v);
+    test_int(v, 10);
+}
+
+void Misc_get_const_var_as_f32_w_out(void) {
+    flecs::world world;
+
+    world.script()
+        .code("export const x: 10")
+        .run();
+
+    float v;
+    world.get_const_var("x", v);
+    test_int(v, 10);
+}
+
+void Misc_get_const_var_as_i32_w_out(void) {
+    flecs::world world;
+
+    world.script()
+        .code("export const x: 10")
+        .run();
+
+    int32_t v;
+    world.get_const_var("x", v);
+    test_int(v, 10);
+}
+
+void Misc_get_const_var_as_u32_w_out(void) {
+    flecs::world world;
+
+    world.script()
+        .code("export const x: 10")
+        .run();
+
+    uint32_t v;
+    world.get_const_var("x", v);
+    test_int(v, 10);
+}
+
+void Misc_get_const_var_struct_w_out(void) {
+    flecs::world world;
+
+    world.component<Position>()
+        .member("x", &Position::x)
+        .member("y", &Position::y);
+
+    world.script()
+        .code("export const x = Position: {10, 20}")
+        .run();
+
+    Position v;
+    world.get_const_var<Position>("x", v, {30, 40});
+    test_int(v.x, 10);
+    test_int(v.y, 20);
+}
+
+void Misc_get_const_var_struct_w_out_not_found(void) {
+    flecs::world world;
+
+    world.component<Position>()
+        .member("x", &Position::x)
+        .member("y", &Position::y);
+
+    Position v;
+    flecs::log::set_level(-4);
+    world.get_const_var<Position>("x", v, {30, 40});
+    test_int(v.x, 30);
+    test_int(v.y, 40);
+}
+
+void Misc_get_const_var_not_found(void) {
+    flecs::world world;
+
+    double v;
+    flecs::log::set_level(-4);
+    world.get_const_var<double>("x", v, 20);
+    test_int(v, 20);
+}
+
+void Misc_get_const_var_not_a_var(void) {
+    flecs::world world;
+
+    world.entity("x");
+
+    double v;
+
+    flecs::log::set_level(-4);
+    world.get_const_var<double>("x", v, 20);
+    test_int(v, 20);
+}
+
+void Misc_get_scoped_const_var(void) {
+    flecs::world world;
+
+    world.script()
+        .code("parent { export const x: 10 }")
+        .run();
+
+    double v = world.get_const_var<double>("parent::x");
+    test_int(v, 10);
+}
+
+struct test_module {
+    static double ScriptVariable;
+
+    test_module(flecs::world& world) {
+        world.script()
+            .code("export const ScriptVariable: 10\n")
+            .run();
+
+        world.get_const_var("ScriptVariable", ScriptVariable, 20.0);
+
+        test_int(ScriptVariable, 10);
+    }
+};
+
+double test_module::ScriptVariable = 0;
+
+void Misc_get_module_const_var(void) {
+    flecs::world world;
+
+    world.import<test_module>();
+
+    test_int(test_module::ScriptVariable, 10);
+}
+
+struct script_module {
+    static double ScriptVariable;
+
+    script_module(flecs::world& world) {
+        world.script()
+            .code("module script_module\n"
+                  "export const ScriptVariable: 10\n")
+            .run();
+
+        world.get_const_var("ScriptVariable", ScriptVariable, 20.0);
+
+        test_int(ScriptVariable, 10);
+    }
+};
+
+double script_module::ScriptVariable = 0;
+
+void Misc_get_module_const_var_from_script_module(void) {
+    flecs::world world;
+
+    world.import<script_module>();
+
+    test_int(script_module::ScriptVariable, 10);
+}
+
+namespace nested {
+
+struct script_module {
+    static double ScriptVariable;
+
+    script_module(flecs::world& world) {
+        world.script()
+            .code("module nested.script_module\n"
+                  "export const ScriptVariable: 10\n")
+            .run();
+
+        world.get_const_var("ScriptVariable", ScriptVariable, 20.0);
+
+        test_int(ScriptVariable, 10);
+    }
+};
+
+double script_module::ScriptVariable = 0;
+
+}
+
+void Misc_get_module_const_var_from_nested_script_module(void) {
+    flecs::world world;
+
+    world.import<nested::script_module>();
+
+    test_int(nested::script_module::ScriptVariable, 10);
+}
+
+namespace nested {
+
+struct test_module {
+    static double ScriptVariable;
+
+    test_module(flecs::world& world) {
+        world.script()
+            .code("export const ScriptVariable: 10\n")
+            .run();
+
+        world.get_const_var("ScriptVariable", ScriptVariable, 20.0);
+
+        test_int(ScriptVariable, 10);
+    }
+};
+
+double test_module::ScriptVariable = 0;
+
+}
+
+void Misc_get_module_const_var_from_nested_module_no_script_module(void) {
+    flecs::world world;
+
+    world.import<nested::test_module>();
+
+    test_int(nested::test_module::ScriptVariable, 10);
+}
