@@ -24,7 +24,7 @@ int main(int, char *[]) {
 
     ecs.progress();
 
-    const Position *p = e.get<Position>();
-    assert(p->x == 11);
-    assert(p->y == 22);
+    const Position& p = e.get<Position>();
+    assert(p.x == 11);
+    assert(p.y == 22);
 }
