@@ -4,6 +4,9 @@
 /* This generated file contains includes for project dependencies */
 #include "script/bake_config.h"
 
+#include <stdio.h>
+#include <stddef.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
